@@ -16,19 +16,35 @@
 
   2002/09/03 - Eliminate (?) false-positives. Original idea from Aaron Sherman.
 
+  2002/11/15 - Updated by Kostya Kortchinsky <kostya.kortchinsky@renater.fr>
+  - ported to SunOS.
 */
-#if !defined(__linux__) && !defined(__FreeBSD__)
+
+#if !defined(__linux__) && !defined(__FreeBSD__) && !defined(__sun)
 main (){ return 0; }
 #else
 #include <stdio.h>
 #include <sys/types.h>
 #include <dirent.h>
+#if defined(__sun)
+#include <procfs.h>
+#include <fcntl.h>
+#endif
 
 //#define MAX_PROCESSES 65535
 
+#if defined(__sun)
+#define FIRST_PROCESS 0
+#else
+#define FIRST_PROCESS 1
+#endif
 #define MAX_PROCESSES 99999
 #define MAX_BUF 1024
+#if defined(__sun)
+#define PS "ps -edf"
+#else
 #define PS "ps auxwww"
+#endif
 
 int psproc [MAX_PROCESSES+1];
 int dirproc[MAX_PROCESSES+1];
@@ -41,7 +57,10 @@ main(int argc, char **argv)
    struct dirent *dir;
    int i, j, retps, retdir;
    int verbose = 0;
-   long ret=0;
+   long ret = 0;
+#if defined(__sun)
+   psinfo_t psbuf;
+#endif
 
    if (!ps || !proc)
    {
@@ -61,17 +80,25 @@ main(int argc, char **argv)
    }
 
    fgets(buf, MAX_BUF, ps); /* Skip header */
+#if defined(__sun)
+   if (!isspace(*buf))
+#else
    if (!isalpha(*buf))
+#endif
    {
      fprintf(stderr, "OooPS!\n");
      exit(2);
    }
-   for (i = 1; i <= MAX_PROCESSES; i++) /* init matrix */
+   for (i = FIRST_PROCESS; i <= MAX_PROCESSES; i++) /* Init matrix */
      psproc[i] = dirproc[i] = 0;
 
    while (fgets(buf, MAX_BUF, ps))
    {
       p = buf;
+#if defined(__sun)
+      while (*p == ' ') /* Skip spaces */
+          p++;
+#endif
       while (*p != ' ') /* Skip USER */
           p++;
       while (*p == ' ') /* Skip spaces */
@@ -82,7 +109,7 @@ main(int argc, char **argv)
          fprintf (stderr, " OooPS, not expected %d value\n");
          exit (2);
       }
-      psproc[ret]=1;
+      psproc[ret] = 1;
    }
    pclose(ps);
 
@@ -91,31 +118,31 @@ main(int argc, char **argv)
       if (!strcmp(dir->d_name, ".") || !strcmp(dir->d_name, "..") ||
          !isdigit(*dir->d_name))
          continue;
-      dirproc[atol(dir->d_name)]=1;
+      dirproc[atol(dir->d_name)] = 1;
    }
    closedir(proc);
 
    /* Brute force */
    strcpy(buf, "/proc/");
    retps = retdir = 0;
-   for (i = 1; i <= MAX_PROCESSES; i++)
+   for (i = FIRST_PROCESS; i <= MAX_PROCESSES; i++)
    {
       snprintf(&buf[6], 6, "%d", i);
       if (!chdir(buf))
       {
-         if (!dirproc[i] && kill(dirproc[i], 0))
+         if (!dirproc[i] && !kill(i, 0))
          {
             retdir++;
             if (verbose)
 	       printf ("PID %5d: not in readdir output\n", i);
          }
-         if (!psproc[i] && kill(psproc[i], 0))
+         if (!psproc[i] && !kill(i, 0))
          {
             retps++;
             if (verbose)
 	       printf ("PID %5d: not in ps output\n", i);
 	 }
-	 if ((!dirproc[i] || !psproc[i]) && (verbose > 1))
+	 if ((!dirproc[i] || !psproc[i]) && !kill(i, 0) && (verbose > 1))
 	 {
 #if defined(__linux__)
 	    j = readlink ("./cwd", path, sizeof(path));
@@ -128,6 +155,17 @@ main(int argc, char **argv)
 	    j = readlink ("./file", path, sizeof(path));
 	    path[(j < sizeof(path)) ? j : sizeof(path) - 1] = 0;
 	    printf ("FILE %5d: %s\n", i, path);
+#elif defined(__sun)
+	    if ((j = open("./psinfo", O_RDONLY)) != -1)
+            {
+               if (read(j, &psbuf, sizeof(psbuf)) == sizeof(psbuf))
+                  printf ("PSINFO %5d: %s\n", i, psbuf.pr_psargs);
+               else
+                  printf ("PSINFO %5d: unknown\n", i);
+               close(j);
+            }
+            else
+               printf ("PSINFO %5d: unknown\n", i);
 #endif
          }
       }
